@@ -27,9 +27,14 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#define _DEBUG_LOG_C
+
 /* vsnprintf() */
 #define _ISOC99_SOURCE
 
+/* facilitynames */
+#define SYSLOG_NAMES
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdarg.h>
@@ -40,6 +45,7 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <syslog.h>
 
 #include <debug/log.h>
 
@@ -87,6 +93,10 @@ struct log_private
    char *filename;		/* filename of log file										*/
    int fd;				/* file descriptor of log file								*/
    struct cache cache;
+
+   /* these fields are only defined if LOG_USE_SYSLOG is set */
+   char *ident;			/* process identifier										*/
+   int syslvl;			/* syslog level												*/
 };
 
 /*
@@ -163,23 +173,36 @@ static char *prefix (struct log_private *priv,const char *filename,int line,cons
    static const char *colors[] = { NULL, RED, WHITE, NULL, GREEN, BROWN, YELLOW };
    static const char *levels[] =
 	 {
-		[LOG_QUIET]   = "  QUIET",
-		[LOG_ERROR]   = "  ERROR",
-		[LOG_WARNING] = "WARNING",
-		[LOG_NORMAL]  = " NORMAL",
-		[LOG_VERBOSE] = "VERBOSE",
-		[LOG_DEBUG]   = "  DEBUG",
-		[LOG_NOISY]   = "  NOISY"
+		[_LOG_QUIET]   = "  QUIET",
+		[_LOG_ERROR]   = "  ERROR",
+		[_LOG_WARNING] = "WARNING",
+		[_LOG_NORMAL]  = " NORMAL",
+		[_LOG_VERBOSE] = "VERBOSE",
+		[_LOG_DEBUG]   = "  DEBUG",
+		[_LOG_NOISY]   = "  NOISY"
 	 };
-   int have_prefix = (!(priv->flags & LOG_DEBUG_PREFIX_ONLY) && priv->level >= LOG_DEBUG) || ((priv->flags & LOG_DEBUG_PREFIX_ONLY) && level >= LOG_DEBUG);
+   static const int syslvl[] =
+	 {
+		[_LOG_QUIET]   = 0,
+		[_LOG_ERROR]   = LOG_ERR,
+		[_LOG_WARNING] = LOG_WARNING,
+		[_LOG_NORMAL]  = LOG_NOTICE,
+		[_LOG_VERBOSE] = LOG_INFO,
+		[_LOG_DEBUG]   = LOG_DEBUG,
+		[_LOG_NOISY]   = LOG_DEBUG
+	 };
+   int have_prefix = (!(priv->flags & LOG_DEBUG_PREFIX_ONLY) && priv->level >= _LOG_DEBUG) || ((priv->flags & LOG_DEBUG_PREFIX_ONLY) && level >= _LOG_DEBUG);
    int i,n = 0;
    char *buf[5];
    size_t len = 0;
 
    if (have_prefix)
 	 {
-		buf[n++] = (priv->flags & LOG_HAVE_COLORS) && isatty (priv->fd) && colors[level] != NULL ?
-		  bprintf (ESC_STR "%sm%s: " ESC_STR "0m",colors[level],levels[level]) : bprintf ("%s: ",levels[level]);
+		buf[n++] = (priv->flags & LOG_HAVE_COLORS) &&
+		  ((priv->flags & LOG_USE_SYSLOG) || isatty (priv->fd)) &&
+		  colors[level] != NULL ?
+		  bprintf (ESC_STR "%sm%s: " ESC_STR "0m",colors[level],levels[level]) :
+		  bprintf ("%s: ",levels[level]);
 	 }
 
    if (priv->flags & LOG_HAVE_LOGFILE)
@@ -234,6 +257,8 @@ static char *prefix (struct log_private *priv,const char *filename,int line,cons
 		mem_free (buf[i]);
 	 }
 
+   priv->syslvl = syslvl[level];
+
    return (buf[4]);
 }
 
@@ -323,7 +348,7 @@ static int flush_cache (struct log_private *priv,const char *filename,int line,c
 
    if (priv->cache.hits)
 	 {
-		char *buf = prefix (priv,filename,line,function,LOG_WARNING);
+		char *buf = prefix (priv,filename,line,function,_LOG_WARNING);
 
 		if (buf == NULL)
 		  goto error;
@@ -362,6 +387,14 @@ static int line_end (struct log_private *priv,const char *filename,int line,cons
 {
    int saved;
 
+   if (priv->flags & LOG_USE_SYSLOG)
+	 {
+		syslog (priv->syslvl,"%s",priv->str);
+		mem_free (priv->str);
+		priv->str = NULL;
+		return (0);
+	 }
+
    if (line_append (priv,"\n") < 0)
 	 return (-1);
 
@@ -408,14 +441,14 @@ static int newline = 1;
 
 static __inline__ void fd_update (struct log_private *priv,int level)
 {
-   if (!(priv->flags & LOG_HAVE_LOGFILE))
-	 priv->fd = level <= LOG_WARNING ? STDERR_FILENO : STDOUT_FILENO;
+   if (!(priv->flags & (LOG_HAVE_LOGFILE | LOG_USE_SYSLOG)))
+	 priv->fd = level <= _LOG_WARNING ? STDERR_FILENO : STDOUT_FILENO;
 }
 
 static int write_string (struct log_private *priv,const char *filename,int line,const char *function,int level,char *str)
 {
    char *s,*tmp;
-   static int prevlevel = LOG_QUIET;
+   static int prevlevel = _LOG_QUIET;
 
    if (level != prevlevel)
 	 {
@@ -468,7 +501,7 @@ error:
 
 static int print_one_line (struct log_private *priv,const char *filename,int line,const char *function,int level,const char *str)
 {
-   while (level != LOG_QUIET && priv->level != LOG_QUIET && level <= priv->level)
+   while (level != _LOG_QUIET && priv->level != _LOG_QUIET && level <= priv->level)
 	 {
 		fd_update (priv,level);
 
@@ -499,6 +532,31 @@ static int print_one_line (struct log_private *priv,const char *filename,int lin
 static struct log_private log_private;
 static int initialized = 0;
 
+static int log_open_syslog (const char *str)
+{
+   char *facility;
+   size_t i;
+
+   if ((log_private.ident = mem_alloc (strlen (str) + 1)) == NULL)
+	 return (-1);
+
+   strcpy (log_private.ident,str);
+
+   if ((facility = strchr (log_private.ident,'.')) == NULL)
+	 return (-1);
+
+   *facility++ = '\0';
+
+   for (i = 0; facilitynames[i].c_name != NULL; i++)
+	 if (!strcmp (facility,facilitynames[i].c_name))
+	   {
+		  openlog (log_private.ident,LOG_PID,facilitynames[i].c_val);
+		  return (0);
+	   }
+
+   return (-1);
+}
+
 int log_open (const char *logfile,int loglevel,int flags)
 {
    if (initialized)
@@ -509,10 +567,13 @@ int log_open (const char *logfile,int loglevel,int flags)
 
    if (logfile != NULL)
 	 {
-		flags |= LOG_HAVE_LOGFILE;
-		flags &= ~LOG_HAVE_COLORS;
+		flags &= ~(LOG_HAVE_COLORS | LOG_HAVE_LOGFILE | LOG_USE_SYSLOG);
+		flags |= !log_open_syslog (logfile) ? LOG_USE_SYSLOG : LOG_HAVE_LOGFILE;
+
+		if (flags & LOG_USE_SYSLOG)
+		  flags &= ~LOG_DETECT_DUPLICATES;
 	 }
-   else flags &= ~LOG_HAVE_LOGFILE;
+   else flags &= ~(LOG_HAVE_LOGFILE | LOG_USE_SYSLOG);
 
    if (flags & LOG_HAVE_LOGFILE)
 	 {
@@ -539,7 +600,7 @@ int log_open (const char *logfile,int loglevel,int flags)
    if (flags & LOG_DETECT_DUPLICATES)
 	 log_private.cache.cache = NULL;
 
-   print_one_line (&log_private,__FILE__,__LINE__,__FUNCTION__,LOG_VERBOSE,"Starting to log output.");
+   print_one_line (&log_private,__FILE__,__LINE__,__FUNCTION__,_LOG_VERBOSE,"Starting to log output.");
 
    initialized = 1;
 
@@ -552,12 +613,18 @@ void log_close (void)
 	 {
 		initialized = 0;
 
-		print_one_line (&log_private,__FILE__,__LINE__,__FUNCTION__,LOG_VERBOSE,"Stopped logging output.");
+		print_one_line (&log_private,__FILE__,__LINE__,__FUNCTION__,_LOG_VERBOSE,"Stopped logging output.");
 
 		if (log_private.flags & LOG_HAVE_LOGFILE)
 		  {
-			 mem_free (log_private.filename);
 			 close (log_private.fd);
+			 mem_free (log_private.filename);
+		  }
+
+		if (log_private.flags & LOG_USE_SYSLOG)
+		  {
+			 closelog ();
+			 mem_free (log_private.ident);
 		  }
 	 }
 }
@@ -572,8 +639,8 @@ int log_reset_stub (const char *filename,int line,const char *function)
 
    if ((log_private.flags & LOG_HAVE_LOGFILE))
 	 {
-		print_one_line (&log_private,filename,line,function,LOG_VERBOSE,"Attempting to reload log file.");
-		print_one_line (&log_private,filename,line,function,LOG_VERBOSE,"Stopped logging output.");
+		print_one_line (&log_private,filename,line,function,_LOG_VERBOSE,"Attempting to reload log file.");
+		print_one_line (&log_private,filename,line,function,_LOG_VERBOSE,"Stopped logging output.");
 
 		close (log_private.fd);
 
@@ -585,8 +652,8 @@ int log_reset_stub (const char *filename,int line,const char *function)
 			 return (-1);
 		  }
 
-		print_one_line (&log_private,filename,line,function,LOG_VERBOSE,"Starting to log output again.");
-		print_one_line (&log_private,filename,line,function,LOG_VERBOSE,"Reload succeeded.");
+		print_one_line (&log_private,filename,line,function,_LOG_VERBOSE,"Starting to log output again.");
+		print_one_line (&log_private,filename,line,function,_LOG_VERBOSE,"Reload succeeded.");
 	 }
 
    return (0);
@@ -603,7 +670,7 @@ int log_vprintf_stub (const char *filename,int line,const char *function,int lev
 		return (-1);
 	 }
 
-   if (level == LOG_QUIET || log_private.level == LOG_QUIET || level > log_private.level)
+   if (level == _LOG_QUIET || log_private.level == _LOG_QUIET || level > log_private.level)
 	 return (0);
 
    if ((buf = vbprintf (format,ap)) == NULL)
@@ -638,7 +705,7 @@ int log_putc_stub (const char *filename,int line,const char *function,int level,
 		return (-1);
 	 }
 
-   if (level == LOG_QUIET || log_private.level == LOG_QUIET || level > log_private.level)
+   if (level == _LOG_QUIET || log_private.level == _LOG_QUIET || level > log_private.level)
 	 return (0);
 
    return (write_string (&log_private,filename,line,function,level,s));
@@ -655,7 +722,7 @@ int log_puts_stub (const char *filename,int line,const char *function,int level,
 		return (-1);
 	 }
 
-   if (level == LOG_QUIET || log_private.level == LOG_QUIET || level > log_private.level)
+   if (level == _LOG_QUIET || log_private.level == _LOG_QUIET || level > log_private.level)
 	 return (0);
 
    if ((s = mem_alloc (sizeof (char) * (strlen (str) + 1))) == NULL)
@@ -678,6 +745,9 @@ int log_flush_stub (const char *filename,int line,const char *function)
 		return (-1);
 	 }
 
+   if (log_private.flags & LOG_USE_SYSLOG)
+	 return (0);
+
    if (flush_cache (&log_private,filename,line,function))
 	 return (-1);
 
